#pragma once

#include <vector>
#include <map>
#include <unordered_map>
#include <cmath>

#include "typedefs.h"

namespace affdex
{    

    /// <summary>
    /// Structure containing the facial expressions scores
    /// </summary>
    struct Expressions
    {
        /// <summary>
        /// Smile score
        /// range of the value is [0, 100]
        /// </summary>
        float smile;
        /// <summary>
        /// Inner brow raise score
        /// range of the value is [0, 100]
        /// </summary>
        float innerBrowRaise;
        /// <summary>
        /// Brow raise score
        /// range of the value is [0, 100]
        /// </summary>
        float browRaise;
        /// <summary>
        /// Brow furrow score
        /// range of the value is [0, 100]
        /// </summary>
        float browFurrow;
        /// <summary>
        /// Nose wrinkler score
        /// range of the value is [0, 100]
        /// </summary>
        float noseWrinkle;
        /// <summary>
        /// Upper lip raiser score
        /// range of the value is [0, 100]
        /// </summary>
        float upperLipRaise;
        /// <summary>
        /// Lip corner depressor score
        /// range of the value is [0, 100]
        /// </summary>
        float lipCornerDepressor;
        /// <summary>
        /// Chin raiser score
        /// range of the value is [0, 100]
        /// </summary>
        float chinRaise;
        /// <summary>
        /// Lip pucker score
        /// range of the value is [0, 100]
        /// </summary>
        float lipPucker;
        /// <summary>
        /// Lip press score
        /// range of the value is [0, 100]
        /// </summary>
        float lipPress;
        /// <summary>
        /// Lip suck score
        /// range of the value is [0, 100]
        /// </summary>
        float lipSuck;
        /// <summary>
        /// Mouth open score
        /// range of the value is [0, 100]
        /// </summary>
        float mouthOpen;
        /// <summary>
        /// Smirk score
        /// range of the value is [0, 100]
        /// </summary>
        float smirk;
        /// <summary>
        /// Eye closure score
        /// range of the value is [0, 100]
        /// </summary>
        float eyeClosure;
        /// <summary>
        /// Attention score
        /// range of the value is [0, 100]
        /// </summary>
        float attention;

        AFFDEXSDK Expressions();
        AFFDEXSDK Expressions(const Expressions &);
    };

    /// <summary>
    /// Structure containing emotions scores
    /// </summary>
    struct Emotions
    {
        /// <summary>
        /// Joy score
        /// range of the value is [0, 100]
        /// </summary>
        float joy;
        /// <summary>
        /// Fear score
        /// range of the value is [0, 100]
        /// </summary>
        float fear;
        /// <summary>
        /// Disgust score
        /// range of the value is [0, 100]
        /// </summary>
        float disgust;
        /// <summary>
        /// Sadness score
        /// range of the value is [0, 100]
        /// </summary>
        float sadness;
        /// <summary>
        /// Anger score
        /// range of the value is [0, 100]
        /// </summary>
        float anger;
        /// <summary>
        /// Suprise score
        /// range of the value is [0, 100]
        /// </summary>
        float surprise;
        /// <summary>
        /// Contempt score
        /// range of the value is [0, 100]
        /// </summary>
        float contempt;
        /// <summary>
        /// Valence score
        /// range of the value is [-100, 100]
        /// </summary>
        float valence;
        /// <summary>
        /// Engagment score
        /// range of the value is [0, 100]
        /// </summary>
        float engagement;

        AFFDEXSDK Emotions();
        AFFDEXSDK Emotions(const Emotions &);
    };

    /// <summary>
    /// Enumeration for the gender values
    /// </summary>
    enum class Gender {
        Unknown,
        Male,
        Female,

    };

    /// <summary>
    ///  Enumeration for the glasses values
    /// </summary>
    enum class Glasses {
        No,
        Yes
    };

    /// <summary>
    ///  Enumeration for the age values
    /// </summary>
    enum Age
    {
        AGE_UNKNOWN = 0,
        AGE_UNDER_18 = 1,
        AGE_18_24 = 2,
        AGE_25_34 = 3,
        AGE_35_44 = 4,
        AGE_45_54 = 5,
        AGE_55_64 = 6,
        AGE_65_PLUS = 7
    };

    /// <summary>
    ///  Enumeration for the ethnicity values
    /// </summary>
    enum Ethnicity
    {
        Unknown = 0,
        Caucasian = 1,
        BlackAfrican = 2,
        SouthAsian = 3,
        EastAsian = 4,
        Hispanic = 5
    };

    struct Appearance
    {
        /// <summary>
        /// Gender
        /// Values {Unknown, Male, Female}
        /// </summary>
        Gender gender;

        /// <summary>
        /// Glasses
        /// Values {No, Yes}
        /// </summary>
        Glasses glasses;

        /// <summary>
        /// Age
        /// Values {AGE_UNKNOWN, AGE_UNDER_18, AGE_18_24,
        /// AGE_25_34, AGE_35_44, AGE_45_54, AGE_55_64, AGE_65_PLUS}
        /// </summary>
        Age age;

        /// <summary>
        /// Ethnicity
        /// Values {Unknown, Caucasian, BlackAfrican, SouthAsian, EastAsian, Hispanic}
        /// </summary>
        Ethnicity ethnicity;

        AFFDEXSDK Appearance();
        AFFDEXSDK Appearance(const Appearance &);
    };

    /// <summary>
    /// Structure containing the angles to represent the head orientation in degrees.
    /// </summary>
    struct Orientation
    {
        /// <summary>
        /// Pitch Angle
        /// </summary>
        float pitch;
        /// <summary>
        /// Yaw Angle
        /// </summary>
        float yaw;
        /// <summary>
        /// Roll Angle
        /// </summary>
        float roll;
        /// <summary>
        /// Constructor.
        /// </summary>
        AFFDEXSDK Orientation(){};
        AFFDEXSDK Orientation(float pitch, float yaw, float roll)
            :
            pitch(pitch), yaw(yaw), roll(roll)
        {};

        /// <summary>
        /// Copy constructor.
        /// </summary>
        AFFDEXSDK Orientation(const Orientation & orientation)
            :
            pitch(orientation.pitch), yaw(orientation.yaw), roll(orientation.roll)
        {};

        /// <summary>
        /// Overloading the equals operator
        /// </summary>
        AFFDEXSDK bool operator==(const Orientation& lhs) const
        {
            double epsilon = 1.0e-2;
            return       (std::fabs(std::fabs(lhs.pitch) - std::fabs(this->pitch)) < epsilon)
                    && (std::fabs(std::fabs(lhs.yaw) - std::fabs(this->yaw)) < epsilon)
                    && (std::fabs(std::fabs(lhs.roll) - std::fabs(this->roll)) < epsilon);
        };
    };

    /// <summary>
    /// Structure containing the measurements
    /// </summary>
    struct Measurements
    {
        Orientation orientation;
        float interocularDistance;
        AFFDEXSDK Measurements();
        AFFDEXSDK Measurements(const Measurements &);
    };

    /// <summary>
    /// Enumeration for the emoji names used by dominantEmoji
    /// </summary>
    enum class Emoji {
        Relaxed = 9786,
        Smiley = 128515,
        Laughing = 128518,
        Kissing = 128535,
        Disappointed = 128542,
        Rage = 128545,
        Smirk = 128527,
        Wink = 128521,
        StuckOutTongueWinkingEye = 128540,
        StuckOutTongue = 128539,
        Flushed = 128563,
        Scream = 128561,
        Unknown = 128528
    };

    /// <summary>
    /// Converts Emoji enum value into string;
    /// <returns> A representation of the  <see cref="Emoji" />.</returns>
    /// </summary>
    static std::string EmojiToString(Emoji emoji) {
        std::string ret = "";
        switch (emoji)
        {
            case Emoji::Relaxed:
                ret = "Relaxed";
                break;
            case Emoji::Smiley:
                ret = "Smiley";
                break;
            case Emoji::Laughing:
                ret = "Laughing";
                break;
            case Emoji::Kissing:
                ret = "Kissing";
                break;
            case Emoji::Disappointed:
                ret = "Disappointed";
                break;
            case Emoji::Rage:
                ret = "Rage";
                break;
            case Emoji::Smirk:
                ret = "Smirk";
                break;
            case Emoji::Wink:
                ret = "Wink";
                break;
            case Emoji::StuckOutTongueWinkingEye:
                ret = "StuckOutTongueWinkingEye";
                break;
            case Emoji::StuckOutTongue:
                ret = "StuckOutTongue";
                break;
            case Emoji::Flushed:
                ret = "Flushed";
                break;
            case Emoji::Scream:
                ret = "Scream";
                break;
            case Emoji::Unknown:
            default:
                ret = "Unknown";
                break;
        };
        return ret;
    }
    
    /// <summary>
    /// Structure containing emoji(s) scores and the value of the most dominant emoji.
    /// </summary>
    struct Emojis
    {
        /// <summary>
        /// relaxed score
        /// range of the value is [0, 100]
        /// </summary>
        float relaxed;

        /// <summary>
        /// smiley score
        /// range of the value is [0, 100]
        /// </summary>
        float smiley;

        /// <summary>
        /// laughing score
        /// range of the value is [0, 100]
        /// </summary>
        float laughing;

        /// <summary>
        /// kiss score
        /// range of the value is [0, 100]
        /// </summary>
        float kissing;
        
        /// <summary>
        /// disappointment score
        /// range of the value is [0, 100]
        /// </summary>
        float disappointed;

        /// <summary>
        /// rage score
        /// range of the value is [0, 100]
        /// </summary>
        float rage;

        /// <summary>
        /// smirk score
        /// range of the value is [0, 100]
        /// </summary>
        float smirk;

        /// <summary>
        /// wink score
        /// range of the value is [0, 100]
        /// </summary>
        float wink;
        
        /// <summary>
        /// tongue out and wink score
        /// range of the value is [0, 100]
        /// </summary>
        float stuckOutTongueWinkingEye;
        
        /// <summary>
        /// tongue out score
        /// range of the value is [0, 100]
        /// </summary>
        float stuckOutTongue;
        
        /// <summary>
        /// flushed score
        /// range of the value is [0, 100]
        /// </summary>
        float flushed;   

        /// <summary>
        /// scream score
        /// range of the value is [0, 100]
        /// </summary>
        float scream;

        /// <summary>
        /// dominant emoji value is the most likely emoji present in the Frame.
        /// </summary>
        Emoji dominantEmoji;

        AFFDEXSDK Emojis();
        AFFDEXSDK Emojis(const Emojis &);          

    };

    /// <summary>
    /// Structure containing basic feature point coordinates.
    /// Coordinate system is an x,y system with the top-left pixel center at ( x=0, y=0 )
    /// and the bottom right pixel at ( x=width-1, y=height-1 )
    /// </summary>
    struct FeaturePoint
    {
        /// <summary>
        /// Point identifier.
        /// </summary>
        int id;

        /// <summary>
        /// X-coordinate of point
        /// </summary>
        float x;


        /// <summary>
        /// Y-coordinate of point
        /// </summary>
        float y;

        /// <summary>
        /// Constructor.
        /// </summary>
        FeaturePoint(int id, float x, float y)
            :
            id(id), x(x), y(y)
        {};

        /// <summary>
        /// Copy constructor.
        /// </summary>
        FeaturePoint(const FeaturePoint & point)
            :
            id(point.id), x(point.x), y(point.y)
        {};
    };

    typedef std::vector<FeaturePoint> VecFeaturePoint;

    /// <summary>
    /// Represents a face found within a processed Frame.
    /// </summary>
    class Face
    {
        friend class DetectorBase;
        friend class SingleFaceDetectorBase;
        friend class MultiFaceDetectorBase;

    public:


        /// <summary>
        /// Initializes a new instance of the <see cref="Face"/> class.
        /// </summary>
        AFFDEXSDK Face();

        /// <summary>
        /// Copy constructor.
        /// </summary>
        /// <param name="face">Face object.</param>
        AFFDEXSDK Face(const Face& face);

        /// <summary>
        /// The face id
        /// </summary>
        FaceId id;

        /// <summary>
        /// The Emotions
        /// </summary>
        Emotions emotions;

        /// <summary>
        /// The Facial Expressions
        /// </summary>
        Expressions expressions;

        /// <summary>
        /// The Face and Head Measurements
        /// </summary>
        Measurements measurements;

        /// <summary>
        /// Facial Appearance
        /// </summary>
        Appearance appearance;

        /// <summary>
        /// Emojis
        /// </summary>
        Emojis emojis;

        /// <summary>
        /// The face points
        /// </summary>
        VecFeaturePoint featurePoints;

    protected:
        Face(MetricsMap metrics);


    };
}
